using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Folder_Helper.Models.Binder;

public class AppSettings
{

    public string? Company { get; set; }
    public string? DefaultNoteType { get; set; }
    public string[]? Exclude { get; set; }
    public string? WorkingDirectoryName { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderAppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings?.Company is null)
            throw new NullReferenceException(nameof(appSettings.Company));
        if (appSettings?.DefaultNoteType is null)
            throw new NullReferenceException(nameof(appSettings.DefaultNoteType));
        if (appSettings?.Exclude is null)
            throw new NullReferenceException(nameof(appSettings.Exclude));
        if (appSettings?.WorkingDirectoryName is null)
            throw new NullReferenceException(nameof(appSettings.WorkingDirectoryName));
        result = new(
            appSettings.Company,
            appSettings.DefaultNoteType,
            appSettings.Exclude,
            appSettings.WorkingDirectoryName
        );
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
#pragma warning disable IL3050, IL2026
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
#pragma warning restore IL3050, IL2026
        result = Get(appSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class BinderAppSettingsSourceGenerationContext : JsonSerializerContext
{
}